@@ -1,5 +1,12 @@
 <?php
-// $Horde: horde/lib/Data.php,v 1.10.2.4 2002/05/11 15:46:54 jan Exp $
+/*
+ * $Horde: horde/lib/Data.php,v 1.10.2.11 2003/01/17 10:22:13 jan Exp $
+ *
+ * Copyright 1999-2003 Jan Schneider <jan@horde.org>
+ *
+ * See the enclosed file COPYING for license information (LGPL). If you
+ * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ */
 
 // Import actions
 
@@ -21,16 +28,26 @@ define('IMPORT_ICALENDAR', 5);
 /** @constant IMPORT_VCARD Import vCards. */
 define('IMPORT_VCARD', 6);
 
+/** @constant IMPORT_TSV Import generic tsv data. */
+define('IMPORT_TSV', 7);
+
+/** @constant IMPORT_MULBERRY Import Mulberry address book data */
+define('IMPORT_MULBERRY', 8);
+
+/** @constant IMPORT_PINE Import Pine address book data. */
+define('IMPORT_PINE', 9);
+
 define('EXPORT_CSV', 100);
 define('EXPORT_ICALENDAR', 101);
 define('EXPORT_VCARD', 102);
+define('EXPORT_TSV', 103);
 
 /**
  * Abstract class to allow data exchange between the Horde applications and
  * external sources
  *
- * @author  Jan Schneider <jan@ammma.de>
- * @version $Revision: 1.10.2.4 $
+ * @author  Jan Schneider <jan@horde.org>
+ * @version $Revision: 1.10.2.11 $
  * @since   Horde 1.3
  * @package horde.data
  */
@@ -117,19 +134,21 @@ class Data_csv extends Data {
             $export = substr($export, 0, -1) . "\n";
         }
 
-        header('Content-Length: ' . strlen($export));
         header('Content-Type: application/csv');
-
-        /* This should force a save file dialog.  According to a note
-         * in MSDN, the suggested filename should NOT be in quotes.
-         *
-         * For IE 5.5, we break the header in a special way that makes
-         * things work. I don't really want to know. */
+        if ($browser->getBrowser() == 'opera') {
+            $filename = strtr($filename, ' ', '_');
+        }
         if ($browser->hasQuirk('break_disposition_header')) {
             header('Content-Disposition: filename=' . $filename);
         } else {
             header('Content-Disposition: attachment; filename=' . $filename);
         }
+        if ($browser->hasQuirk('cache_ssl_downloads')) {
+            header('Expires: 0');
+            header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
+            header('Pragma: public');
+        }
+        header('Content-Length: ' . strlen($export));
 
         echo $export;
     }
@@ -340,6 +359,105 @@ class Data_rfc2425 extends Data {
 
         return $date_arr;
     }
+}
+
+class Data_tsv extends Data {
+
+    function importFile($filename, $header=false, $delimiter = "\t")
+    {
+        $fp = fopen($filename, 'r');
+        if (!$fp) {
+            return false;
+        }
+
+        $contents = preg_split( "/(\n|\r|\r\n)/", rtrim(fread($fp, filesize($filename))));
+
+        fclose($fp);
+
+        $data = array();
+
+        if ($header) {
+            $head = explode($delimiter, array_shift($contents));
+        }
+
+        foreach ($contents as $line) {
+            $line = explode($delimiter, $line);
+            if (!isset($head)) {
+                $data[] = $line;
+            } else {
+                $newline = array();
+                for ($i = 0; $i < count($head); $i++) {
+                    //if (!empty($line[$i]))
+                    $newline[$head[$i]] = empty($line[$i]) ? '' : $line[$i];
+                }
+                $data[] = $newline;
+            }
+        }
+
+        return $data;
+    }
+
+    function import()
+    {
+    }
+
+    function export()
+    {
+    }
+
+    function exportFile($filename, $data, $header=false, $delimiter = "\t")
+    {
+        if (!is_array($data) || count($data) == 0) {
+            return;
+        }
+
+        include_once HORDE_BASE . '/lib/Browser.php';
+        $browser = new Browser();
+
+        $export = '';
+
+        if ($header) {
+            $head = current($data);
+            while (list($key,) = each($head)) {
+                if (!empty($key)) {
+                    $export .= '"' . addslashes($key) . '"';
+                }
+                $export .= "\t";
+            }
+            $export = substr($export, 0, -1) . "\n";
+        }
+
+        foreach ($data as $row) {
+            foreach ($row as $cell) {
+                if (!empty($cell) || $cell === 0) {
+                    $export .= '"' . addslashes($cell) . '"';
+                }
+                $export .= "\t";
+            }
+            $export = substr($export, 0, -1) . "\n";
+        }
+
+        header('Content-Type: application/csv');
+        if ($browser->getBrowser() == 'opera') {
+            $filename = strtr($filename, ' ', '_');
+        }
+        if ($browser->hasQuirk('break_disposition_header')) {
+            header('Content-Disposition: filename=' . $filename);
+        } else {
+            header('Content-Disposition: attachment; filename=' . $filename);
+        }
+        if ($browser->hasQuirk('cache_ssl_downloads')) {
+            header('Expires: 0');
+            header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
+            header('Pragma: public');
+        }
+        header('Content-Length: ' . strlen($export));
+
+        echo $export;
+    }
+
+    function mapDate()
+    {
+    }
 
 }
-?>
