@@ -1,7 +1,15 @@
 <?php
-// $Horde: horde/lib/Horde.php,v 1.118.2.20 2002/05/22 10:08:58 jan Exp $
+/*
+ * $Horde: horde/lib/Horde.php,v 1.118.2.39 2003/01/17 00:58:35 slusarz Exp $
+ *
+ * Copyright 1999-2003 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2003 Jon Parise <jon@horde.org>
+ *
+ * See the enclosed file COPYING for license information (LGPL). If you
+ * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ */
 
-// Horde Common Actions
+// Horde Framework Actions
 /** @constant NOOP The null action/noop. */
 define('NOOP', 0);
 /** @constant NO_ACTION The null action/noop. */
@@ -12,7 +20,6 @@ define('HORDE_UPDATE_PREFS', 1);
 /** @constant UPDATE_PREFS Update preferences. */
 define('UPDATE_PREFS', 1);
 
-// Horde Framework Actions
 /** @constant HORDE_LOGIN The login action. */
 define('HORDE_LOGIN', 50);
 
@@ -44,27 +51,30 @@ define('HORDE_ERROR', 4);
 
 
 /**
- * If called with a filename, register that filename to be deleted at
- * request shutdown. If called with no arguments, unlink all files
- * registered. The first time it is called, it initializes the array
- * of files, and registers itself as a shutdown function - no need to
- * do so manually.
- * The second parameter allows the unregistering of previously
- * registered filenames.
+ * Removes given elements at request shutdown.
+ * If called with a filename will delete that file at request shutdown.
+ * If called with a directory will remove that directory and all files in
+ * that directory at request shutdown.
+ * If called with no arguments, unlink all elements registered.
+ * The first time it is called, it initializes the array and registers itself
+ * as a shutdown function - no need to do so manually.
+ * The second parameter allows the unregistering of previously registered
+ * elements.
  *
- * @access private
+ * @access public
  *
- * @param optional string $filename The filename to be deleted at the end of
- *                                  the request.
- * @param optional string $register If true, then register the file for
- *                                  deletion, otherwise, unregister it.
+ * @param optional string $filename   The filename to be deleted at the end of
+ *                                    the request.
+ * @param optional boolean $register  If true, then register the element for
+ *                                    deletion, otherwise, unregister it.
  */
 function _fileCleanup($filename = false, $register = true)
 {
-    static $files;
+    static $dirs, $files;
 
-    /* Initialization of variables and shutdown functions */
+    /* Initialization of variables and shutdown functions. */
     if (!isset($files) || !is_array($files)) {
+        $dirs = array();
         $files = array();
         register_shutdown_function('_fileCleanup');
     }
@@ -72,14 +82,34 @@ function _fileCleanup($filename = false, $register = true)
     if ($register) {
         if (!$filename) {
             foreach ($files as $file => $val) {
+                /* Delete file */
                 if ($val && @file_exists($file)) {
                     @unlink($file);
                 }
             }
+            foreach ($dirs as $dir => $val) {
+                /* Delete directories */
+                if ($val && @file_exists($dir)) {
+                    /* Make sure directory is empty. */
+                    $dir_class = dir($dir);
+                    while (false !== ($entry = $dir_class->read())) {
+                        if ($entry != '.' && $entry != '..') {
+                            @unlink($dir . '/' . $entry);
+                        }
+                    }
+                    $dir_class->close();
+                    @rmdir($dir);
+                }
+            }
         } else {
-            $files[$filename] = true;
+            if (@is_dir($filename)) {
+                $dirs[$filename] = true;
+            } else {
+                $files[$filename] = true;
+            }
         }
     } else {
+        $dirs[$filename] = false;
         $files[$filename] = false;
     }
 }
@@ -90,7 +120,7 @@ function _fileCleanup($filename = false, $register = true)
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Jon Parise <jon@horde.org>
- * @version $Revision: 1.118.2.20 $
+ * @version $Revision: 1.118.2.39 $
  * @since   Horde 1.3
  * @package horde
  */
@@ -145,15 +175,16 @@ class Horde {
             include_once 'Log.php';
 
             // Try to make sure that we can log messages somehow.
-            if (!isset($conf['log']['type']) ||
-                !isset($conf['log']['name']) ||
-                !isset($conf['log']['ident']) ||
-                !isset($conf['log']['params'])) {
+            if (!array_key_exists('log', $conf) ||
+                !array_key_exists('type', $conf['log']) ||
+                !array_key_exists('name', $conf['log']) ||
+                !array_key_exists('ident', $conf['log']) ||
+                !array_key_exists('params', $conf['log'])) {
                 Horde::fatal(new PEAR_Error('Horde is not correctly configured to log error messages. You must configure at least a text file log in horde/config/horde.php.'), __FILE__, __LINE__, false);
             }
 
-            $logger = Log::singleton($conf['log']['type'], $conf['log']['name'],
-                                     $conf['log']['ident'], $conf['log']['params']);
+            $logger = &Log::singleton($conf['log']['type'], $conf['log']['name'],
+                                      $conf['log']['ident'], $conf['log']['params']);
 
             if (!$logger || !is_object($logger) || !is_subclass_of($logger, 'log')) {
                 if (PEAR::isError($logger)) {
@@ -175,7 +206,7 @@ class Horde {
             }
         }
 
-        $app = isset($GLOBALS['registry']) ? $GLOBALS['registry']->getApp() : 'horde';
+        $app = array_key_exists('registry', $GLOBALS) ? $GLOBALS['registry']->getApp() : 'horde';
         $message = '[' . $app . '] ' . $message . ' [on line ' . $line . ' of "' . $file . '"]';
 
         $logger->log($message, $priority);
@@ -189,13 +220,13 @@ class Horde {
      * @access public
      *
      * @param object PEAR_Error $error  An error object with debug information.
-     * @param integer $file The file in which the error occured.
-     * @param integer $line The line on which the error occured.
-     * @param optional boolean $log  Log this message via Horde::logMesage()?
+     * @param integer $file             The file in which the error occured.
+     * @param integer $line             The line on which the error occured.
+     * @param optional boolean $log     Log this message via Horde::logMesage()?
      */
     function fatal($error, $file, $line, $log = true)
     {
-        @include_once HORDE_BASE . '/lib/Text.php';
+        include_once HORDE_BASE . '/lib/Text.php';
 
         $errortext = _("<b>A fatal error has occurred:</b>") . "<br /><br />\n";
         if (is_object($error) && method_exists($error, 'getMessage')) {
@@ -271,8 +302,9 @@ HTML;
      *
      * @access public
      *
-     * @param string  $uri The URI to be modified.
-     * @param boolean $full Generate a full (http://server/path/) URL.
+     * @param string  $uri                   The URI to be modified.
+     * @param boolean $full                  Generate a full
+     *                                       (http://server/path/) URL.
      * @param boolean $always_append_session Tack on the session ID even if
      *                                       cookies are present.
      *
@@ -283,7 +315,7 @@ HTML;
         $protocol = 'http';
 
         if ($full) {
-            global $conf, $HTTP_SERVER_VARS, $registry;
+            global $conf, $registry;
 
             /* Store connection parameters in local variables. */
             $server_port = $registry->getParam('server_port');
@@ -292,12 +324,14 @@ HTML;
             if ($conf['use_ssl'] == 1) {
                 $protocol = 'https';
             } else if ($conf['use_ssl'] == 2) {
-                if ((isset($HTTP_SERVER_VARS['HTTPS']) && $HTTP_SERVER_VARS['HTTPS'] == 'on') || getenv('SSL_PROTOCOL_VERSION')) {
+                if (Horde::usingSSLConnection()) {
                     $protocol = 'https';
                 }
             }
 
-            if ($server_port != 80 && $server_port != 443) {
+            /* If using non-standard ports, add the port to the URL. */
+            if ((($protocol == 'http') && ($server_port != 80)) ||
+                (($protocol == 'https') && ($server_port != 443))) {
                 $server_name .= ':' . $server_port;
             }
 
@@ -319,7 +353,7 @@ HTML;
         }
 
         if ($always_append_session ||
-            !isset($GLOBALS['HTTP_COOKIE_VARS'][session_name()])) {
+            !array_key_exists(session_name(), $_COOKIE)) {
             $url = Horde::addParameter($url, urlencode(session_name()) . '=' . session_id());
         }
 
@@ -336,13 +370,20 @@ HTML;
      * @param string $parameter The name=value pair to add.
      *
      * @return string The modified URL.
+     *
+     * @since Horde 2.1
      */
     function addParameter($url, $parameter)
     {
-        if (strstr($url, $parameter) === false) {
+        if (!empty($parameter) && strstr($url, $parameter) === false) {
+            if (substr($parameter, 0, 1) == '?') {
+                $parameter = substr($parameter, 1);
+            }
+
             $pos = strpos($url, '?');
             if ($pos !== false) {
-                $url = substr_replace($url, $parameter . '&', $pos + 1, 0);
+                $url = substr_replace($url, $parameter . ini_get('arg_separator.output'),
+                                      $pos + 1, 0);
             } else {
                 $url .= '?' . $parameter;
             }
@@ -350,14 +391,38 @@ HTML;
         return $url;
     }
 
+    /**
+     * Removes a name=value pair from a URL.
+     *
+     * @access public
+     *
+     * @param string $url       The URL to modify.
+     * @param array $parameter  The array of parameters to remove.
+     *
+     * @return string  The modified URL.
+     *
+     * @since Horde 2.2
+     */
+    function removeParameter($url, $parameter = array())
+    {
+        foreach ($parameter as $value) {
+            $url = preg_replace("/" . $value . "\=\w+\&?(?:amp;)?/", '', $url);
+        }
+
+        /* If there are no more parameters left, or the last parameter was
+           removed, remove the trailing '?' or '&'. */
+        return rtrim($url, '&?');
+    }
+
     /**
      * Return a session-id-ified version of $uri, using the current
      * application's webroot setting.
      *
      * @access public
      *
-     * @param string  $uri The URI to be modified.
-     * @param boolean $full Generate a full (http://server/path/) URL.
+     * @param string  $uri                   The URI to be modified.
+     * @param boolean $full                  Generate a full
+     *                                       (http://server/path/) URL.
      * @param boolean $always_append_session Tack on the session ID even if
      *                                       cookies are present.
      *
@@ -401,7 +466,7 @@ HTML;
         }
         if (!empty($status)) {
             $ret .= ' onmouseout="window.status=\'\';" onmouseover="window.status=\'' .
-                htmlspecialchars(addslashes($status), ENT_QUOTES, Lang::getCharset()) . '\'; return true;"';
+                @htmlspecialchars(addslashes($status), ENT_QUOTES, Lang::getCharset()) . '\'; return true;"';
         }
         if (!empty($class)) {
             $ret .= " class=\"$class\"";
@@ -410,7 +475,7 @@ HTML;
             $ret .= " target=\"$target\"";
         }
         if (!empty($title)) {
-            $ret .= ' title="' . htmlspecialchars($title, ENT_QUOTES, Lang::getCharset()) . '"';
+            $ret .= ' title="' . @htmlspecialchars($title, ENT_QUOTES, Lang::getCharset()) . '"';
         }
 
         return "$ret>";
@@ -421,10 +486,10 @@ HTML;
      *
      * @access public
      *
-     * @param string $url The full URL to be linked to
-     * @param string $status An optional JavaScript mouse-over string
-     * @param string $class The CSS class of the link
-     * @param string $target The window target to point this link too
+     * @param string $url     The full URL to be linked to
+     * @param string $status  An optional JavaScript mouse-over string
+     * @param string $class   The CSS class of the link
+     * @param string $target  The window target to point this link too
      * @param string $onclick JavaScript action for the 'onclick' event.
      * @param string $title   The link title (tooltip)
      */
@@ -438,10 +503,11 @@ HTML;
      *
      * @access public
      *
-     * @param string  $uri the URI to be modified
-     * @param boolean $full Generate a full (http://server/path/) URL.
+     * @param string  $uri                   the URI to be modified
+     * @param boolean $full                  Generate a full
+     *                                       (http://server/path/) URL.
      * @param boolean $always_append_session Tack on the session ID even if
-     *                               cookies are present.
+     *                                       cookies are present.
      */
     function purl($uri, $full = false, $always_append_session = false)
     {
@@ -457,12 +523,10 @@ HTML;
      */
     function selfURL($query_string = false)
     {
-        global $HTTP_SERVER_VARS;
+        $url = $_SERVER['PHP_SELF'];
 
-        $url = $HTTP_SERVER_VARS['PHP_SELF'];
-
-        if ($query_string && !empty($HTTP_SERVER_VARS['QUERY_STRING'])) {
-            $url .= '?' . $HTTP_SERVER_VARS['QUERY_STRING'];
+        if ($query_string && !empty($_SERVER['QUERY_STRING'])) {
+            $url .= '?' . $_SERVER['QUERY_STRING'];
         }
 
         return Horde::url($url);
@@ -506,13 +570,14 @@ HTML;
      * @access public
      *
      * @param          string $src  The image file.
-     * @param optional string $attr Any extra parameters for the image tag.
+     * @param optional string $attr Any additional attributes for the image tag.
      * @param optional string $dir  The root graphics directory.
      *
      * @return string The full image tag.
      */
     function img($src, $attr = '', $dir = null)
     {
+        /* If no directory has been specified, get it from the registry. */
         if ($dir === null) {
             global $registry;
             $dir = $registry->getParam('graphics');
@@ -540,9 +605,9 @@ HTML;
      *
      * @access public
      *
-     * @param string $src  The image file.
-     * @param string $attr Any extra parameters for the image tag.
-     * @param string $dir  The root graphics directory.
+     * @param          string $src  The image file.
+     * @param optional string $attr Any additional attributes for the image tag.
+     * @param optional string $dir  The root graphics directory.
      */
     function pimg($src, $attr = null, $dir = null)
     {
@@ -554,8 +619,9 @@ HTML;
      *
      * @access public
      *
-     * @param  string $var The string to un-quote, if necessary.
-     * @return string      $var, minus any magic quotes.
+     * @param  string $var  The string to un-quote, if necessary.
+     *
+     * @return string       $var, minus any magic quotes.
      */
     function dispelMagicQuotes(&$var)
     {
@@ -578,25 +644,64 @@ HTML;
 
     /**
      * Get a form variable from GET or POST data, stripped of magic
-     * quotes if necessary.
+     * quotes if necessary. If the variable is somehow set in both the
+     * GET data and the POST data, the value from the POST data will
+     * be returned and the GET value will be ignored.
      *
      * @access public
      *
-     * @param string $var The name of the form variable to look for.
-     * @param string $default (optional) The value to return if the
-     *                                   variable is not there.
+     * @param string $var       The name of the form variable to look for.
+     * @param string $default   (optional) The value to return if the
+     *                          variable is not there.
      *
      * @return string     The cleaned form variable, or $default.
      */
     function getFormData($var, $default = null)
     {
-        if (isset($GLOBALS['HTTP_POST_VARS'][$var])) {
-            return Horde::dispelMagicQuotes($GLOBALS['HTTP_POST_VARS'][$var]);
-        } elseif (isset($GLOBALS['HTTP_GET_VARS'][$var])) {
-            return Horde::dispelMagicQuotes($GLOBALS['HTTP_GET_VARS'][$var]);
-        } else {
-            return $default;
-        }
+        return ($val = Horde::getPost($var)) !== null
+            ? $val : Horde::getGet($var, $default);
+    }
+
+    /**
+     * Get a form variable from GET data, stripped of magic quotes if
+     * necessary. This function will NOT return a POST variable.
+     *
+     * @access public
+     *
+     * @param string $var       The name of the form variable to look for.
+     * @param string $default   (optional) The value to return if the
+     *                          variable is not there.
+     *
+     * @return string     The cleaned form variable, or $default.
+     *
+     * @since Horde 2.2
+     */
+    function getGet($var, $default = null)
+    {
+        return (array_key_exists($var, $_GET))
+            ? Horde::dispelMagicQuotes($_GET[$var])
+            : $default;
+    }
+
+    /**
+     * Get a form variable from POST data, stripped of magic quotes if
+     * necessary. This function will NOT return a GET variable.
+     *
+     * @access public
+     *
+     * @param string $var       The name of the form variable to look for.
+     * @param string $default   (optional) The value to return if the
+     *                          variable is not there.
+     *
+     * @return string     The cleaned form variable, or $default.
+     *
+     * @since Horde 2.2
+     */
+    function getPost($var, $default = null)
+    {
+        return (array_key_exists($var, $_POST))
+            ? Horde::dispelMagicQuotes($_POST[$var])
+            : $default;
     }
 
     /**
@@ -605,12 +710,12 @@ HTML;
      *
      * @access public
      *
-     * @return string    A directory name which can be used for temp files
-     *                   or false if one could not be found
+     * @return string   A directory name which can be used for temp files.
+     *                  Returns false if one could not be found.
      */
     function getTempDir()
     {
-        $tmp_locations = array('/tmp', '/var/tmp', 'c:\temp');
+        $tmp_locations = array('/tmp', '/var/tmp', 'c:\temp', 'c:\windows\temp', 'c:\winnt\temp');
 
         /* If one has been specifically set, then use that */
         if (!empty($GLOBALS['conf']['tmpdir'])) {
@@ -648,11 +753,15 @@ HTML;
      *
      * @access public
      *
-     * @param string $prefix  Prefix to make the temporary name more recognizable
-     * @param optional boolean $delete Delete the file at the end of the request?
-     * @param optional string $dir Directory to create the temporary file in
-     * @return string         containing the full path-name to the temporary file
-     *                        or false if a temp file could not be created
+     * @param string $prefix            Prefix to make the temporary name more
+     *                                  recognizable.
+     * @param optional boolean $delete  Delete the file at the end of the
+     *                                  request?
+     * @param optional string $dir      Directory to create the temporary file
+     *                                  in.
+     *
+     * @return string   Returns the full path-name to the temporary file.
+     *                  Returns false if a temp file could not be created.
      */
     function getTempFile($prefix = 'Horde', $delete = true, $dir = false)
     {
@@ -678,6 +787,126 @@ HTML;
             return $tmp_file;
         }
     }
-}
 
+    /**
+     * Create a temporary directory in the system's temporary directory.
+     *
+     * @access public
+     *
+     * @param optional boolean $delete  Delete the temporary directory at the
+     *                                  end of the request?
+     *
+     * @return string       The pathname to the new temporary directory.
+     *                      Returns false if directory not created.
+     *
+     * @since Horde 2.2
+     */
+    function createTempDir($delete = true)
+    {
+        $temp_dir = Horde::getTempDir();
+        if (empty($temp_dir)) return false;
+
+        /* Get the first 8 characters of a random string to use as a temporary
+           directory name. */
+        do {
+            $temp_dir .= '/' . substr(md5(uniqid(rand())), 0, 8);
+        } while (file_exists($temp_dir));
+
+        $old_umask = umask(0000);
+        if (!mkdir($temp_dir, 0700)) {
+            $temp_dir = false;
+        } else {
+            if ($delete) {
+                _fileCleanup($temp_dir);
+            }
+        }
+        umask($old_umask);
+
+        return $temp_dir;
+    }
+
+    /**
+     * Determine if we are using a Secure (SSL) connection.
+     *
+     * @access public
+     *
+     * @return boolean      True if using SSL, false if not.
+     */
+    function usingSSLConnection()
+    {
+        return ((array_key_exists('HTTPS', $_SERVER) && $_SERVER['HTTPS'] == 'on') ||
+                getenv('SSL_PROTOCOL_VERSION'));
+    }
+
+    /**
+     * Start output compression, if requested.
+     *
+     * @access public
+     *
+     * @since Horde 2.2
+     */
+    function compressOutput()
+    {
+        static $started;
+
+        if (isset($started)) {
+            return;
+        }
+
+        global $browser, $conf;
+
+        /* Netscape =< 4 is so buggy with compression that we just turn it
+           completely off for those browsers. */
+        if ($conf['compress_pages'] &&
+            (ini_get('zlib.output_compression') != 1) &&
+            (($browser->getBrowser() != 'mozilla') ||
+             ($browser->getMajor() > 4))) {
+            /* Compress output if requested. */
+            ob_start('ob_gzhandler');
+        }
+
+        $started = true;
+    }
+
+    /**
+     * If there is a custom session handler, set it up now.
+     *
+     * @access public
+     *
+     * @since Horde 2.3
+     */
+    function setupSessionHandler()
+    {
+        global $conf;
+
+        $type = !empty($conf['sessionhandler']['type']) ? $conf['sessionhandler']['type'] : 'none';
+
+        if ($type == 'external') {
+            $calls = $conf['sessionhandler']['params'];
+            session_set_save_handler($calls['open'],
+                                     $calls['close'],
+                                     $calls['read'],
+                                     $calls['write'],
+                                     $calls['destroy'],
+                                     $calls['gc']);
+        } elseif ($type != 'none') {
+            global $_session_handler;
+            require_once HORDE_BASE . '/lib/SessionHandler.php';
+            $_session_handler = &SessionHandler::singleton($conf['sessionhandler']['type']);
+            if (!empty($_session_handler) &&
+                !PEAR::isError($_session_handler)) {
+                ini_set('session.save_handler', 'user');
+                session_set_save_handler(array($_session_handler, 'open'),
+                                         array($_session_handler, 'close'),
+                                         array($_session_handler, 'read'),
+                                         array($_session_handler, 'write'),
+                                         array($_session_handler, 'destroy'),
+                                         array($_session_handler, 'gc'));
+            } else {
+                Horde::fatal(PEAR::raiseError('Horde is unable to correctly start the custom session handler.'), __FILE__, __LINE__, false);
+            }
+        }
+    }
+
+}
 ?>
