@@ -1,11 +1,11 @@
 <?php
-/* $Horde: horde/lib/Maintenance.php,v 1.2.2.6 2002/06/09 03:26:49 slusarz Exp $
-*
-* Copyright 2001-2002 Michael Slusarz <slusarz@bigworm.colorado.edu>
-*
-* See the enclosed file COPYING for license information (LGPL).  If you
-* did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
-*/
+/* $Horde: horde/lib/Maintenance.php,v 1.2.2.10 2003/01/03 12:48:39 jan Exp $
+ *
+ * Copyright 2001-2003 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ *
+ * See the enclosed file COPYING for license information (LGPL).  If you
+ * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ */
 
 /** @const MAINTENANCE_YEARLY Do task yearly (First login after/on January 1). */
 define('MAINTENANCE_YEARLY', 1);
@@ -37,7 +37,7 @@ $GLOBALS['intervals'] = &$intervals;
 * maintenance operations run upon login to Horde applications.
 *
 * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
-* @version $Revision: 1.2.2.6 $
+* @version $Revision: 1.2.2.10 $
 * @since   Horde 1.3.5
 * @package horde
 */
@@ -120,7 +120,8 @@ class Maintenance {
         $this->last_login = $prefs->getValue('last_login');
 
         /* Store whether the confirm maintenance page has been shown yet. */
-        if (Horde::getFormData('confirm_maintenance')) {
+        if (Horde::getFormData('confirm_maintenance') ||
+            !$prefs->getValue('confirm_maintenance')) {
             $this->confirm_page = true;
         }
 
@@ -139,7 +140,7 @@ class Maintenance {
      */
     function runMaintenance()
     {
-        global $prefs, $registry, $HTTP_SERVER_VARS;
+        global $prefs, $registry;
 
         /* Get the list of tasks available during this login.
            If no tasks present, we can return now. */
@@ -154,8 +155,8 @@ class Maintenance {
         if (!$this->confirm_page) {
             $confirmation = $this->confirmationMaintenance();
             if (!empty($confirmation)) {
-                if (!empty($HTTP_SERVER_VARS['QUERY_STRING'])) {
-                    $getvars = urlencode($HTTP_SERVER_VARS['QUERY_STRING']);
+                if (!empty($_SERVER['QUERY_STRING'])) {
+                    $getvars = urlencode($_SERVER['QUERY_STRING']);
                 } else {
                     $getvars = '';
                 }
@@ -277,8 +278,10 @@ class Maintenance {
         /* Determine which of these tasks have been confirmed. */
         foreach ($available as $value) {
             if (Horde::getFormData($value . '_confirm') ||
-                ($prefs->isLocked($value) && $prefs->getValue($value)))
+                (($prefs->isLocked($value) || !$prefs->getValue('confirm_maintenance'))
+                 && $prefs->getValue($value))) {
                 $return_array[] = $value;
+            }
         }
 
         return $return_array;
